#![windows_subsystem = "windows"]
use bevy::{prelude::*, window::PrimaryWindow, winit::WinitSettings};
use bevy_egui::{
    egui::{self},
    EguiContexts, EguiPlugin,
};

#[cfg(windows)]
const LINE_ENDING: &str = "\r\n";
#[cfg(not(windows))]
const LINE_ENDING: &str = "\n";

#[derive(Resource)]
struct InputText {
    value: String,
}

#[derive(Resource)]
struct OutputText {
    value: String,
}

fn main() {
    App::new()
        .insert_resource(WinitSettings::desktop_app())
        .insert_resource(InputText {
            value: "".to_string(),
        })
        .insert_resource(OutputText {
            value: "".to_string(),
        })
        .add_plugins(DefaultPlugins)
        .add_plugins(EguiPlugin)
        .add_systems(Startup, set_window_title)
        .add_systems(Update, ui_system)
        .add_systems(Update, process_text)
        .run();
}

fn set_window_title(mut window_query: Query<&mut Window, With<PrimaryWindow>>) {
    if let Ok(mut window) = window_query.get_single_mut() {
        window.title = "dsort".to_string();
    }
}

fn process_text(input_text: Res<InputText>, mut output_text: ResMut<OutputText>) {
    let mut arr = input_text.value.lines().collect::<Vec<_>>();
    arr.sort_by(|&a, &b| vsort::compare(a, b));
    output_text.value = arr.join(LINE_ENDING);
}

fn ui_system(
    mut contexts: EguiContexts,
    mut input_text: ResMut<InputText>,
    mut output_text: ResMut<OutputText>,
) {
    contexts.ctx_mut().set_visuals(egui::Visuals::light());
    egui::CentralPanel::default().show(contexts.ctx_mut(), |ui| {
        let window_size = ui.available_size();
        let height = window_size[1];
        ui.with_layout(
            egui::Layout::left_to_right(egui::Align::Center).with_cross_justify(true),
            |ui| {
                egui::ScrollArea::vertical()
                    .id_source("left")
                    .show(ui, |ui| {
                        ui.add_sized(
                            [ui.available_width() / 2., height],
                            egui::TextEdit::multiline(&mut input_text.value),
                        );
                    });

                egui::ScrollArea::vertical()
                    .id_source("right")
                    .show(ui, |ui| {
                        ui.add_sized(
                            [ui.available_width(), height],
                            egui::TextEdit::multiline(&mut output_text.value),
                        );
                    });
            },
        );
    });
}
